@@ -1,5 +1,6 @@
 import numpy as np
 from . import config as ttconf
+from . import TreeTimeUnknownError
 from .distribution import Distribution
 from .utils import clip
 from .config import FFT_FWHM_GRID_SIZE
@@ -164,64 +165,89 @@ class NodeInterpolator (Distribution):
         dt = max(branch_interp.one_mutation*0.005, min(node_interp.fwhm, branch_interp.fwhm)/fft_grid_size)
         b_effsupport = branch_interp.effective_support
         n_effsupport = node_interp.effective_support
+        b_support_range = b_effsupport[1]-b_effsupport[0]
+        n_support_range = n_effsupport[1]-n_effsupport[0]
+        # compare the support of the node distribution to the width of the branch length distribution
+        ratio = n_support_range/branch_interp.fwhm
 
-        tmax = 2*max(b_effsupport[1]-b_effsupport[0], n_effsupport[1]-n_effsupport[0])
-
-        Tb = np.arange(b_effsupport[0], b_effsupport[0] + tmax + dt, dt)
-        if inverse_time:
-            Tn = np.arange(n_effsupport[0], n_effsupport[0] + tmax + dt, dt)
-            Tmin = node_interp.xmin
-            Tmax = ttconf.MAX_BRANCH_LENGTH
+        if ratio < 1.0/fft_grid_size and 4.0*dt > node_interp.fwhm:
+            ## node distribution is much narrower than the branch distribution, proceed as if
+            # node distribution is a delta distribution with the peak 4 full-width-half-maxima
+            # away from the nominal peak to avoid slicing the relevant range to zero
+            log_scale_node_interp = node_interp.integrate(return_log=True, a=node_interp.xmin,b=node_interp.xmax,n=max(100, len(node_interp.x))) #probability of node distribution
+            if inverse_time:
+                x = branch_interp.x + max(n_effsupport[0], node_interp._peak_pos - 4.0*node_interp.fwhm)
+                dist = Distribution(x, branch_interp(x - node_interp._peak_pos) - log_scale_node_interp,
+                                    min_width=max(node_interp.min_width, branch_interp.min_width), is_log=True)
+            else:
+                x = - branch_interp.x + min(n_effsupport[1], node_interp._peak_pos + 4.0*node_interp.fwhm)
+                dist = Distribution(x, branch_interp(branch_interp.x) - log_scale_node_interp,
+                                    min_width=max(node_interp.min_width, branch_interp.min_width), is_log=True)
+            return dist
+        elif ratio > fft_grid_size and 4*dt > branch_interp.fwhm:
+            raise ValueError("ERROR: Unexpected behavior: branch distribution is much narrower than the node distribution.")
         else:
-            Tn = np.arange(n_effsupport[1] - tmax, n_effsupport[1] + dt, dt)
-            Tmin = -ttconf.MAX_BRANCH_LENGTH
-            Tmax = node_interp.xmax
+            tmax = 2*max(b_support_range, n_support_range)
 
-        raw_len = len(Tb)
-        fft_len = 2*raw_len
+            Tb = np.arange(b_effsupport[0], b_effsupport[0] + tmax + dt, dt)
+            if inverse_time:
+                Tn = np.arange(n_effsupport[0], n_effsupport[0] + tmax + dt, dt)
+                Tmin = node_interp.xmin
+                Tmax = ttconf.MAX_BRANCH_LENGTH
+            else:
+                Tn = np.arange(n_effsupport[1] - tmax, n_effsupport[1] + dt, dt)
+                Tmin = -ttconf.MAX_BRANCH_LENGTH
+                Tmax = node_interp.xmax
 
-        fftb = branch_interp.fft(Tb, n=fft_len)
-        fftn = node_interp.fft(Tn, n=fft_len, inverse_time=inverse_time)
-        if inverse_time:
-            fft_res = np.fft.irfft(fftb*fftn, fft_len)[:raw_len]
-            Tres = Tn + Tb[0]
-        else:
-            fft_res = np.fft.irfft(fftb*fftn, fft_len)[::-1]
-            fft_res = fft_res[raw_len:]
-            Tres = Tn - Tb[0]
+            raw_len = len(Tb)
+            fft_len = 2*raw_len
 
-        # determine region in which we can trust the FFT convolution and avoid
-        # inaccuracies due to machine precision. 1e-13 seems robust
-        ind = fft_res>fft_res.max()*1e-13
-        res = -np.log(fft_res[ind]) + branch_interp.peak_val + node_interp.peak_val - np.log(dt)
-        Tres_cropped = Tres[ind]
+            fftb = branch_interp.fft(Tb, n=fft_len)
+            fftn = node_interp.fft(Tn, n=fft_len, inverse_time=inverse_time)
+            if inverse_time:
+                fft_res = np.fft.irfft(fftb*fftn, fft_len)[:raw_len]
+                Tres = Tn + Tb[0]
+            else:
+                fft_res = np.fft.irfft(fftb*fftn, fft_len)[::-1]
+                fft_res = fft_res[raw_len:]
+                Tres = Tn - Tb[0]
 
-        # extrapolate the tails exponentially: use margin last data points
-        margin = np.minimum(3, Tres_cropped.shape[0]//3)
-        if margin<1 or len(res)==0:
-            import ipdb; ipdb.set_trace()
-        else:
-            left_slope = (res[margin]-res[0])/(Tres_cropped[margin]-Tres_cropped[0])
-            right_slope = (res[-1]-res[-margin-1])/(Tres_cropped[-1]-Tres_cropped[-margin-1])
+            # determine region in which we can trust the FFT convolution and avoid
+            # inaccuracies due to machine precision. 1e-13 seems robust
+            ind = fft_res>fft_res.max()*1e-13
+            res = -np.log(fft_res[ind]) + branch_interp.peak_val + node_interp.peak_val - np.log(dt)
 
-        # only extrapolate on the left when the slope is negative and we are not on the boundary
-        if Tmin<Tres_cropped[0] and left_slope<0:
-            Tleft = np.linspace(Tmin, Tres_cropped[0],10)[:-1]
-            res_left = res[0] + left_slope*(Tleft - Tres_cropped[0])
-        else:
-            Tleft, res_left = [], []
+            Tres_cropped = Tres[ind]
 
-        # only extrapolate on the right when the slope is positive and we are not on the boundary
-        if Tres_cropped[-1]<Tmax and right_slope>0:
-            Tright = np.linspace(Tres_cropped[-1], Tmax,10)[1:]
-            res_right = res[-1] + right_slope*(Tright - Tres_cropped[-1])
-        else: #otherwise
-            Tright, res_right = [], []
+            # extrapolate the tails exponentially: use margin last data points
+            margin = np.minimum(3, Tres_cropped.shape[0]//3)
+            if margin<1 or len(res)==0:
+                raise TreeTimeUnknownError("Error: Unexpected behavior detected in FFT function. "
+                                     "No valid points left after reducing to plausible region.\n\n"
+                        "If you see this error please let us know by filling an issue at:\n"
+                        "https://github.com/neherlab/treetime/issues")
+            else:
+                left_slope = (res[margin]-res[0])/(Tres_cropped[margin]-Tres_cropped[0])
+                right_slope = (res[-1]-res[-margin-1])/(Tres_cropped[-1]-Tres_cropped[-margin-1])
 
-        # instantiate the new interpolation object and return
-        return cls(np.concatenate((Tleft,Tres_cropped,Tright)),
-                   np.concatenate((res_left, res, res_right)),
-                   is_log=True, kind='linear', assume_sorted=True)
+            # only extrapolate on the left when the slope is negative and we are not on the boundary
+            if Tmin<Tres_cropped[0] and left_slope<0:
+                Tleft = np.linspace(Tmin, Tres_cropped[0],10)[:-1]
+                res_left = res[0] + left_slope*(Tleft - Tres_cropped[0])
+            else:
+                Tleft, res_left = [], []
+
+            # only extrapolate on the right when the slope is positive and we are not on the boundary
+            if Tres_cropped[-1]<Tmax and right_slope>0:
+                Tright = np.linspace(Tres_cropped[-1], Tmax,10)[1:]
+                res_right = res[-1] + right_slope*(Tright - Tres_cropped[-1])
+            else: #otherwise
+                Tright, res_right = [], []
+
+            # instantiate the new interpolation object and return
+            return cls(np.concatenate((Tleft,Tres_cropped,Tright)),
+                    np.concatenate((res_left, res, res_right)),
+                    is_log=True, kind='linear', assume_sorted=True)
 
     @classmethod
     def convolve(cls, node_interp, branch_interp, max_or_integral='integral',
