@@ -1,9 +1,8 @@
-from __future__ import division, print_function, absolute_import
 import sys
 from os.path import isfile
 from collections import defaultdict
 import numpy as np
-from Bio import SeqRecord, Seq, AlignIO, SeqIO
+from Bio import AlignIO, SeqIO
 from . import config as ttconf
 from . import MissingDataError
 from .seq_utils import seq2array, guess_alphabet, alphabets
@@ -91,7 +90,7 @@ class SequenceData(object):
         self._ref = None
         self.likely_alphabet = None
         self.compressed_to_full_sequence_map = None
-        self.multiplicity = None
+        self._multiplicity = None
         self.is_sparse = None
         self.convert_upper = convert_upper
         self.compress = compress
@@ -159,7 +158,7 @@ class SequenceData(object):
                     except:
                         continue
 
-        if type(in_aln) is MultipleSeqAlignment:
+        if isinstance(in_aln, MultipleSeqAlignment):
             # check whether the alignment is consistent with a nucleotide alignment.
             self._aln = {}
             for s in in_aln:
@@ -271,8 +270,13 @@ class SequenceData(object):
             self._ref = seq2array(in_ref, fill_overhangs=False, word_length=self.word_length)
             self.full_length = self._ref.shape[0]
             self.compressed_to_full_sequence_map = None
-            self.multiplicity = None
+            self._multiplicity = None
 
+    def multiplicity(self, mask=None):
+        if mask is None:
+            return self._multiplicity
+        else:
+            return self._multiplicity*mask
 
     def check_alphabet(self, seqs):
         self.likely_alphabet = guess_alphabet(seqs)
@@ -310,7 +314,7 @@ class SequenceData(object):
         """
 
         if not self.compress: #
-            self.multiplicity = np.ones(self.full_length, dtype=float)
+            self._multiplicity = np.ones(self.full_length, dtype=float)
             self.full_to_compressed_sequence_map = np.arange(self.full_length)
             self.compressed_to_full_sequence_map = {p:np.array([p]) for p in np.arange(self.full_length)}
             self._compressed_length = self._full_length
@@ -402,9 +406,9 @@ class SequenceData(object):
 
 
         # count how many times each column is repeated in the real alignment
-        self.multiplicity = np.zeros(len(alignment_patterns))
+        self._multiplicity = np.zeros(len(alignment_patterns))
         for p, pos in alignment_patterns.values():
-            self.multiplicity[p]=len(pos)
+            self._multiplicity[p]=len(pos)
 
         # create the compressed alignment as a dictionary linking names to sequences
         tmp_compressed_alignment = np.array(compressed_aln_transpose).T
@@ -421,7 +425,7 @@ class SequenceData(object):
             self.compressed_to_full_sequence_map[val[0]]=np.array(val[1], dtype=int)
 
         self.logger("SequenceData: constructed compressed alignment...", 1)
-        self._compressed_length = len(self.multiplicity)
+        self._compressed_length = len(self._multiplicity)
         return ttconf.SUCCESS
 
 
@@ -463,7 +467,6 @@ class SequenceData(object):
         """
         if self.ref is None:
             raise TypeError("SequenceData: sparse sequences can only be constructed when a reference sequence is defined")
-        sparse_seq = {}
 
         compressed_nonref_positions = self.full_to_compressed_sequence_map[self.nonref_positions]
         compressed_nonref_values = sequence[compressed_nonref_positions]
@@ -499,7 +502,7 @@ class SequenceData(object):
         else:
             return tmp_seq
 
-    def differences(self, seq1, seq2, seq1_compressed=True, seq2_compressed=True):
+    def differences(self, seq1, seq2, seq1_compressed=True, seq2_compressed=True, mask=None):
         diffs = []
         if self.is_sparse:
             if seq1_compressed: seq1 = self.compressed_to_sparse_sequence(seq1)
@@ -514,7 +517,11 @@ class SequenceData(object):
         else:
             if seq1_compressed: seq1 = self.compressed_to_full_sequence(seq1)
             if seq2_compressed: seq2 = self.compressed_to_full_sequence(seq2)
-            diff_pos = np.where(seq1 != seq2)[0]
+            if mask is None:
+                diff_pos = np.where(seq1 != seq2)[0]
+            else:
+                diff_pos = np.where((seq1 != seq2)&(mask>0))[0]
+
             for pos in diff_pos:
                 diffs.append((seq1[pos], pos, seq2[pos]))
 
